/******************************************************************************
 *
 * nttime.c - NT 64bits timestamps handling code. Part of the DirTools project.
 *
 * $Revision: 1.1.1.1 $
 *
 * Copyright (c) 2001-2004 Olivier Castan
 *
 * This program/include file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program/include file is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (in the main directory of the Linux-NTFS
 * distribution in the file COPYING); if not, write to the Free Software
 * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * format : indent -br -ce -cdw -nut -i5
 *
 *****************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdint.h>

#include "nttime.h"

/**********************************************************************
 * time_prn
 *
 *
 *
 *
 *
 *
 *********************************************************************/
void
time_prn (FILE * fOutput, uint64_t ntTime)
{
#define STARTYEAR 1601
#define SECTODAY (24L * 3600L)
#define NANOTOSEC 10000000
#define LEAPYEAR(year) (!((year) % 4) && (((year) % 100) || ! ((year) % 400)))
#define YEARSIZE(year) (LEAPYEAR(year)? 366 : 365)

     unsigned long long sec;
     unsigned long tm_nano;
     unsigned long dayclock;
     unsigned long dayno;
     int tm_sec, tm_min, tm_hour, tm_year, tm_mon, tm_mday;
     static unsigned int _wytab[2][12] =
          { {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
     {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
     };

     sec = ntTime / NANOTOSEC;
     tm_nano = ntTime % NANOTOSEC;
     tm_year = STARTYEAR;
     dayclock = (unsigned long) (sec % SECTODAY);
     dayno = (unsigned long) (sec / SECTODAY);
     tm_sec = dayclock % 60;
     tm_min = (dayclock % 3600) / 60;
     tm_hour = dayclock / 3600;
     while (dayno >= YEARSIZE (tm_year)) {
          dayno -= YEARSIZE (tm_year);
          tm_year++;
     }
     tm_mon = 0;
     while (dayno >= _wytab[LEAPYEAR (tm_year)][tm_mon]) {
          dayno -= _wytab[LEAPYEAR (tm_year)][tm_mon];
          tm_mon++;
     }
     tm_mon++;
     tm_mday = dayno + 1;
     fprintf (fOutput, "%.2u/%.2u/%u %.2u:%.2u:%.2u.%.7lu",
              tm_mday, tm_mon, tm_year, tm_hour, tm_min, tm_sec, tm_nano);
}
